"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var index_1 = require("../WhitePage/index");
var dom_1 = require("../../utils/dom");
var index_2 = require("../AbstractWhiteboard/index");
var toolbar_items_1 = require("../../toolbar/toolbar-items");
var LeftArrowIcon = require('../../assets/bx-left-arrow.svg');
var RightArrowIcon = require('../../assets/bx-right-arrow.svg');
var FinishIcon = require('../../assets/finish.svg');
var RollbackIcon = require('../../assets/rollback.svg');
var prefix = 'fcw-board';
var Whiteboard = (function (_super) {
    __extends(Whiteboard, _super);
    function Whiteboard() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.mode = 'master';
        return _this;
    }
    Whiteboard.prototype.destroy = function () {
        _super.prototype.destroy.call(this);
    };
    Whiteboard.prototype.init = function () {
        this.imgsContainer = dom_1.createDivWithClassName(prefix + "-imgs", this.target);
        this.pagesContainer = dom_1.createDivWithClassName(prefix + "-pages", this.target);
        this.initMaster();
        this.snapHistory.push(this.captureSnap(false));
        this.emitSnapshot();
    };
    Whiteboard.prototype.initMaster = function () {
        var _this = this;
        this.isInitialized = true;
        var prevToolbarItem = {
            icon: LeftArrowIcon,
            name: 'prev-flip-arrow',
            tooltipText: 'Prev',
            onClick: function () {
                var nextPageIndex = _this.visiblePageIndex - 1 < 0 ? _this.pages.length - 1 : _this.visiblePageIndex - 1;
                document.querySelectorAll('.fc-whiteboard-indicator-current').forEach(function (e) {
                    e.innerHTML = "" + (nextPageIndex + 1);
                });
                _this.onPageChange(nextPageIndex);
            }
        };
        var indicatorContainer = document.createElement('div');
        indicatorContainer.className = 'fc-whiteboard-indicator-container';
        var indicatorCurrent = document.createElement('span');
        indicatorCurrent.className = 'fc-whiteboard-indicator-current';
        indicatorCurrent.innerHTML = "" + (this.visiblePageIndex + 1);
        indicatorContainer.appendChild(indicatorCurrent);
        indicatorContainer.appendChild(document.createTextNode("/" + this.sources.length));
        var indicatorItem = {
            name: 'indicator',
            onRender: function () { return indicatorContainer; }
        };
        var nextToolbarItem = {
            icon: RightArrowIcon,
            name: 'next-flip-arrow',
            tooltipText: 'Next',
            onClick: function () {
                var nextPageIndex = _this.visiblePageIndex + 1 > _this.pages.length - 1 ? 0 : _this.visiblePageIndex + 1;
                document.querySelectorAll('.fc-whiteboard-indicator-current').forEach(function (e) {
                    e.innerHTML = "" + (nextPageIndex + 1);
                });
                _this.onPageChange(nextPageIndex);
            }
        };
        var finishItem = {
            icon: FinishIcon,
            name: 'finish',
            tooltipText: 'Finish',
            onClick: function () {
                _this.emit({
                    event: 'finish',
                    id: _this.id,
                    target: 'whiteboard'
                });
            }
        };
        var rollbackItem = {
            icon: RollbackIcon,
            name: 'rollback',
            tooltipText: 'Rollback',
            shortcut: 'ESC',
            onClick: function () {
                _this.rollbackSnap();
            }
        };
        this.sources.forEach(function (source) {
            var page = new index_1.WhitePage({ imgSrc: source }, {
                mode: _this.mode,
                whiteboard: _this,
                parentContainer: _this.pagesContainer,
                extraToolbarItems: [
                    toolbar_items_1.separatorToolbarItem,
                    prevToolbarItem,
                    indicatorItem,
                    nextToolbarItem,
                    toolbar_items_1.separatorToolbarItem,
                    finishItem,
                    rollbackItem
                ]
            });
            page.container.style.visibility = 'hidden';
            _this.pages.push(page);
        });
        this.initSiema();
    };
    Whiteboard.prototype.onPageChange = function (nextPageIndex) {
        if (this.visiblePageIndex === nextPageIndex) {
            return;
        }
        this.siema.goTo(nextPageIndex);
        this.visiblePageIndex = nextPageIndex;
        this.pages.forEach(function (page, i) {
            if (nextPageIndex === i) {
                page.show();
            }
            else {
                page.hide();
            }
        });
        this.emit({
            event: 'borderChangePage',
            id: this.id,
            target: 'whiteboard',
            border: this.captureSnap()
        });
    };
    Whiteboard.prototype.emitSnapshot = function () {
        var _this = this;
        var innerFunc = function () {
            _this.emit({
                event: 'borderSnap',
                id: _this.id,
                target: 'whiteboard',
                border: _this.captureSnap(false)
            });
        };
        this.emitInterval = setInterval(function () {
            innerFunc();
        }, this.snapInterval);
        setTimeout(innerFunc, 500);
    };
    return Whiteboard;
}(index_2.AbstractWhiteboard));
exports.Whiteboard = Whiteboard;
